local combobulator = {}


local projectId = "Combobulator"
local fs = love.filesystem
local mkdir = fs.createDirectory
local format = string.format
local json = require "lunajson"


local folders = {
	[".lua"] = "scripts",
	[".py"]  = "scripts",
	[".bmp"] = "images",
	[".tga"] = "images",
	[".png"] = "images",
	[".jpg"] = "images",
	[".jpeg"]= "images",
	[".gif"] = "images",
	[".mp4"] = "video",
	[".ts"]  = "video",
	[".mov"] = "video",
	[".flv"] = "video",
	[".mkv"] = "video",
	[".avi"] = "video",
	[".webm"]= "video",
	[".mp3"] = "audio",
	[".aac"] = "audio",
	[".ogg"] = "audio",
	[".wav"] = "audio",
	[".txt"] = "text",
	[".csv"] = "text",
}
local combobPaths = {}
local combobFiles = {}


local function combobFile(path)
	if not path then return nil, nil end
	file = path:match("[^/\\.]+%.-%w+$")
	if not file then return nil, nil end
	local ext = file:match("(%.%w+)$") or ""
	local fid = (folders[ext] or "misc").."/"..file
	local data = combobPaths[path]
	if data then
		return combobPaths[path].file, combobPaths[path].ext
	else
		local index = combobFiles[file]
		if not index then
			combobFiles[file] = 1
		else
			index = index + 1
			combobFiles[file] = index
			file = file:sub(1, file:len() - ext:len()).."-"..index..ext
		end
		combobPaths[path] = { file = file, ext = ext }
		return file, ext
	end
end


function fileExists( path )
	if path then
		local fh = io.open(path, 'r')
		if fh then io.close(fh) end
		return fh ~= nil
	end
	return false
end


function fileCopy( path, newPath )
	if fileExists(path) then
		local fh = io.open(path, 'rb')
		if fh then
			local data = fh:read('*a')
			io.close(fh)
			if data then
				fh = io.open(newPath, 'wb')
				if fh then
					local result = fh:write(data)
					io.close(fh)
					return result
				end
			end
		end
	end
	return false
end


function combobulator.export(data, exportName, deck, schedule, sfx)
	local check, data = pcall(json.decode, data)
	if not check then return false, 'Error decoding scene collection data' end
	if not exportName or exportName == "" then return false, 'A valid package name is required (alphanumeric + _)' end
	exportName = exportName:gsub("%W", "_")
	if fs.getInfo("Combobulator/"..exportName) then return false, format('Package name already exists : %s', exportName) end

	local msgPath = fs.getSaveDirectory()
	local targetPath = "[U_COMBOBULATOR_PATH]/"
	local combobPath = projectId.."/"..exportName.."/"
	local fullCombobPath = msgPath.."/"..combobPath
	local folderList = { "scripts", "images", "video", "audio", "text", "misc" }
	if sfx then folderList[#folderList + 1] = "sfx" end
	for _, dir in ipairs(folderList) do
		if not mkdir(combobPath..dir) then return false, "Could not create combob folder: "..dir end
	end

	data.name = exportName
	data = json.encode(data, "___null___")

	local deckCopied = false
	if deck and deck ~= "" then
		local f = fs.read("Backup/"..deck)
		if f then deckCopied = fs.write(combobPath.."deck.txt", deck.."\n"..f) end
	end

	local scheduleCopied = false
	if schedule and schedule ~= "" then
		local f = fs.read("Scheduler/"..schedule)
		if f then scheduleCopied = fs.write(combobPath.."schedule.txt", schedule.."\n"..f) end
	end

	local sfxFailed = 0
	if sfx then sfxFailed = fs.rCopy("Sounds", combobPath.."sfx") end

	local total = 0
	local success = 0
	local failed = 0
	local file, ext, folder

	-- replace msgPath references
	local i, s, f = 1, nil, nil
	while true do
		local s, f = data:find(msgPath, i, true)
		if not s then break end
		data = data:sub(1, s - 1).."[U_MESSAGE_PATH]"..data:sub(f + 1)
		i = s
	end

	-- replace source paths
	for path in string.gmatch(data, "%\"C*:*/[^%\"]-%\"") do
		path = path:sub(2, -2)
		if fileExists(path) then
			total = total + 1
			file, ext = combobFile(path)
			folder = folders[ext] or "misc"
			if file and fileCopy( path, fullCombobPath..folder.."/"..file) then
				data = data:gsub(path:gsub("(%W)", "%%%1"), targetPath..folder.."/"..file)
				success = success + 1
			else
				failed = failed + 1
			end
		end
	end

	local ok = fs.write(combobPath..exportName..".json", data)
	if not ok then return false, string.format("Error creating export file : %s", exportName..".json") end

	return true, { total = total, success = success, failed = failed, deckCopied = deckCopied, scheduleCopied = scheduleCopied, sfxFailed = sfxFailed }
end


return combobulator