function scriptTransition()
	if scrSwitch.busy then
		obs.script_log(obs.LOG_INFO, "Transition already in progress")
		return false
	end
	scrSwitch.busy = true
	local sceneName = "UPD_TRANSITION"
	local sceneNameA = currentSceneName() or ""
	local sceneNameB = vParams.scene or ""
	if sceneNameB == "_previous" and prevScene then sceneNameB = prevScene end
	local baseW, baseH
	local scene, sceneSource, sourceA, sourceB
	local function releaseAll(animating)
		if not animating then scrSwitch.busy = false end
		if scene then obs.obs_scene_release(scene) end
		if sceneSource then obs.obs_source_release(sceneSource) end
		if sourceA then obs.obs_source_release(sourceA) end
		if sourceB then obs.obs_source_release(sourceB) end
	end

	if sceneNameA == sceneName or sceneNameB == sceneName then
		obs.script_log(obs.LOG_INFO, format("Aborting - current scene : %s", sceneName))
		return false
	end

	sceneSource = obs.obs_get_source_by_name(sceneName)
	if not sceneSource then
		if debug then obs.script_log(obs.LOG_INFO, format("Creating scene : %s", sceneName)) end
		scene = obs.obs_scene_create(sceneName)
		if scene then sceneSource = obs.obs_get_source_by_name(sceneName) end
	else
		if debug then obs.script_log(obs.LOG_INFO, format("Found scene : %s", sceneName)) end
		scene = obs.obs_scene_from_source(sceneSource)
		if scene then obs.obs_scene_addref(scene) end
	end
	if not scene or not sceneSource then
		if debug then obs.script_log(obs.LOG_INFO, format("Error accessing scene : %s", sceneName)) end
		releaseAll()
		return false
	end

	emptyScene(scene)

	sourceA = obs.obs_get_source_by_name(sceneNameA)
	if not sourceA then
		if debug then obs.script_log(obs.LOG_INFO, format("Can't find sceneA : %s", sceneNameA)) end
		releaseAll()
		return false
	end
	if debug then obs.script_log(obs.LOG_INFO, format("Adding sceneA : %s", sceneNameA)) end
	local itemA = obs.obs_scene_add(scene, sourceA)
	if not itemA then
		if debug then obs.script_log(obs.LOG_INFO, format("Can't add sceneA : %s", sceneNameA)) end
		releaseAll()
		return false
	end

	sourceB = obs.obs_get_source_by_name(sceneNameB)
	if not sourceB then
		if debug then obs.script_log(obs.LOG_INFO, format("Can't find sceneB : %s", sceneNameB)) end
		releaseAll()
		return false
	end
	local itemB = obs.obs_scene_add(scene, sourceB)
	if not itemB then
		if debug then obs.script_log(obs.LOG_INFO, format("Can't add sceneB : %s", sceneNameB)) end
		releaseAll()
		return false
	end

	baseW = obs.obs_source_get_width(sourceA)
	baseH = obs.obs_source_get_height(sourceA)

	if debug then obs.script_log(obs.LOG_INFO, format("Canvas : %d x %d", baseW, baseH)) end

	obs.obs_sceneitem_set_alignment(itemB, obsAlignment.center)
	local pos = obs.vec2()
	pos.x, pos.y = baseW * 0.5, baseH * 0.5
	obs.obs_sceneitem_set_pos(itemB, pos)
	obs.obs_sceneitem_set_bounds_type(itemB, obsBoundsType.stretch)
	local size = obs.vec2()
	size.x, size.y = 1, 1
	obs.obs_sceneitem_set_bounds(itemB, size) -- w / h

	-- switch to transition scene
	local transitions = obs.obs_frontend_get_transitions()
	local transition = findSource(transitions, "Cut")
	if transition then obs.obs_frontend_set_current_transition(transition) end
	obs.source_list_release(transitions)
	prevScene = currentSceneName()

	obs.timer_add(
		function()
			obs.obs_frontend_set_current_scene(sceneSource)

			obs.script_log(obs.LOG_INFO, "Transition ready to animate")
			cmd = format(
				"animate\ttag=UPDtrans\tscene=%s\titem=%s\tsteps=100\tinterval=5\tw=%d\th=%d\tr=360.5\teasing=EaseOutElastic\n",
				sceneName, sceneNameB, baseW, baseH
			)
			listen(cmd)
			obs.timer_add(
				function()
					if (animTag["UPDtrans"] or 0) > 0 then
						obs.script_log(obs.LOG_INFO, "Transition in progress...")
					else
						local targetScene = obs.obs_get_source_by_name(sceneNameB)
						if targetScene then
							obs.obs_frontend_set_current_scene(targetScene)
							obs.obs_source_release(targetScene)
						end
						scrSwitch.busy = false
						obs.script_log(obs.LOG_INFO, "Transition complete")
						obs.remove_current_callback()
					end
				end,
				500
			)
			releaseAll("animating")
			obs.remove_current_callback()
		end,
		200
	)

end