function scriptTransition()
	if scrSwitch.busy then
		obs.script_log(obs.LOG_INFO, "Transition already in progress")
		return false
	end
	scrSwitch.busy = true
	local sceneName = "UPD_TRANSITION"
	local sceneNameA = currentSceneName() or ""
	local sceneNameB = vParams.scene or ""
	if sceneNameB == "_previous" and prevScene then sceneNameB = prevScene end
	local groupNameL = "UPD_TRANS_GROUP_LEFT"
	local groupNameR = "UPD_TRANS_GROUP_RIGHT"
	local baseW, baseH
	local scene, sceneSource, sourceA, sourceB, groupL, groupR
	local function releaseAll(animating)
		if not animating then scrSwitch.busy = false end
		if scene then obs.obs_scene_release(scene) end
		if sceneSource then obs.obs_source_release(sceneSource) end
		if sourceA then obs.obs_source_release(sourceA) end
		if sourceB then obs.obs_source_release(sourceB) end
		if groupL then obs.obs_source_release(groupL) end
		if groupR then obs.obs_source_release(groupR) end
	end

	if sceneNameA == sceneName or sceneNameB == sceneName then
		obs.script_log(obs.LOG_INFO, format("Aborting - current scene : %s", sceneName))
		return false
	end

	sceneSource = obs.obs_get_source_by_name(sceneName)
	if not sceneSource then
		if debug then obs.script_log(obs.LOG_INFO, format("Creating scene : %s", sceneName)) end
		scene = obs.obs_scene_create(sceneName)
		if scene then sceneSource = obs.obs_get_source_by_name(sceneName) end
	else
		if debug then obs.script_log(obs.LOG_INFO, format("Found scene : %s", sceneName)) end
		scene = obs.obs_scene_from_source(sceneSource)
		if scene then obs.obs_scene_addref(scene) end
	end
	if not scene or not sceneSource then
		if debug then obs.script_log(obs.LOG_INFO, format("Error accessing scene : %s", sceneName)) end
		releaseAll()
		return false
	end

	emptyScene(scene)

	sourceA = obs.obs_get_source_by_name(sceneNameA)
	if not sourceA then
		if debug then obs.script_log(obs.LOG_INFO, format("Can't find sceneA : %s", sceneNameA)) end
		releaseAll()
		return false
	end
	if debug then obs.script_log(obs.LOG_INFO, format("Adding sceneA : %s", sceneNameA)) end
	local itemA = obs.obs_scene_add(scene, sourceA)

	sourceB = obs.obs_get_source_by_name(sceneNameB)
	if not sourceB then
		if debug then obs.script_log(obs.LOG_INFO, format("Can't find sceneB : %s", sceneNameB)) end
		releaseAll()
		return false
	end

	groupL = obs.obs_get_source_by_name(groupNameL)
	if not groupL then
		if debug then obs.script_log(obs.LOG_INFO, format("Creating group : %s", groupNameL)) end
		groupL = obs.obs_source_create("group", groupNameL, nil, nil)
		if not groupL then
			if debug then obs.script_log(obs.LOG_INFO, format("Can't create group : %s", groupNameL)) end
			releaseAll()
			return false
		end
	end
	local groupItemL = obs.obs_scene_add(scene, groupL)
	if not groupItemL then
		if debug then obs.script_log(obs.LOG_INFO, format("Can't add group to scene : %s", groupNameL)) end
		releaseAll()
		return false
	end
	local groupSceneL = obs.obs_sceneitem_group_get_scene(groupItemL)
	if not groupSceneL then
		if debug then obs.script_log(obs.LOG_INFO, format("Can't access group : %s", groupNameL)) end
		releaseAll()
		return false
	end
	emptyScene(groupSceneL)

	groupR = obs.obs_get_source_by_name(groupNameR)
	if not groupR then
		if debug then obs.script_log(obs.LOG_INFO, format("Creating group : %s", groupNameR)) end
		groupR = obs.obs_source_create("group", groupNameR, nil, nil)
		if not groupR then
			if debug then obs.script_log(obs.LOG_INFO, format("Can't create group : %s", groupNameR)) end
			releaseAll()
			return false
		end
	end
	local groupItemR = obs.obs_scene_add(scene, groupR)
	if not groupItemR then
		if debug then obs.script_log(obs.LOG_INFO, format("Can't add group to scene : %s", groupNameR)) end
		releaseAll()
		return false
	end
	local groupSceneR = obs.obs_sceneitem_group_get_scene(groupItemR)
	if not groupSceneR then
		if debug then obs.script_log(obs.LOG_INFO, format("Can't access group : %s", groupNameR)) end
		releaseAll()
		return false
	end
	emptyScene(groupSceneR)

	if debug then obs.script_log(obs.LOG_INFO, format("Adding sceneA : %s", sceneNameA)) end
	local itemBL = obs.obs_scene_add(groupSceneL, sourceB)
	local itemBR = obs.obs_scene_add(groupSceneR, sourceB)
	baseW = obs.obs_source_get_width(sourceA)
	baseH = obs.obs_source_get_height(sourceA)

	if debug then obs.script_log(obs.LOG_INFO, format("Canvas : %d x %d", baseW, baseH)) end

	local crp = obs.obs_sceneitem_crop()
	crp.left, crp.top, crp.right, crp.bottom = 0, 0, baseW * 0.5, 0
	obs.obs_sceneitem_set_crop(groupItemL, crp)
	crp.left, crp.right = baseW * 0.5, 0
	obs.obs_sceneitem_set_crop(groupItemR, crp)
	local pos = obs.vec2()
	pos.x, pos.y = -baseW * 0.5 - 5, 0
	obs.obs_sceneitem_set_pos(groupItemL, pos)
	pos.x, pos.y = baseW + 5, 0
	obs.obs_sceneitem_set_pos(groupItemR, pos)

	-- switch to transition scene
	local transitions = obs.obs_frontend_get_transitions()
	local transition = findSource(transitions, "Cut")
	if transition then obs.obs_frontend_set_current_transition(transition) end
	obs.source_list_release(transitions)
	prevScene = currentSceneName()

	obs.timer_add(
		function()
			obs.obs_frontend_set_current_scene(sceneSource)

			obs.script_log(obs.LOG_INFO, "Transition ready to animate")
			cmd = format(
				"animate\ttag=UPDtrans\tscene=%s\titem=%s\tsteps=100\tinterval=10\tx=%d\teasing=easein\n"..
				"animate\ttag=UPDtrans\tscene=%s\titem=%s\tsteps=100\tinterval=10\tx=%d\teasing=easein\n",
				sceneName, groupNameL, 0,
				sceneName, groupNameR, baseW * 0.5
			)
			listen(cmd)
			obs.timer_add(
				function()
					if (animTag["UPDtrans"] or 0) > 0 then
						obs.script_log(obs.LOG_INFO, "Transition in progress...")
					else
						local targetScene = obs.obs_get_source_by_name(sceneNameB)
						if targetScene then
							obs.obs_frontend_set_current_scene(targetScene)
							obs.obs_source_release(targetScene)
						end
						scrSwitch.busy = false
						obs.script_log(obs.LOG_INFO, "Transition complete")
						obs.remove_current_callback()
					end
				end,
				500
			)
			releaseAll("animating")
			obs.remove_current_callback()
		end,
		200
	)

end