-- Verion : 0.1
-- Released : 2019-07-11


function scriptTransition()
	

	-- Check the busy flag
	if scrSwitch.busy then
		obs.script_log(obs.LOG_INFO, "Transition already in progress")
		return false
	end
	scrSwitch.busy = true

	
	-- The scene to perform the transition in
	local transSceneName = "UPD_TRANSITION"

	
	-- The current and target scenes
	local sceneNameA = currentSceneName() or ""
	local sceneNameB = vParams.scene or ""
	if sceneNameB == "_previous" and prevScene then sceneNameB = prevScene end

	
	-- This transition creates 2 copies of the target scene - one for each half on the effect.
	-- The only way I've found (so far) to do this is to create 2 groups and put the scene in each group.
	local groupNameL = "UPD_TRANS_GROUP_LEFT"
	local groupNameR = "UPD_TRANS_GROUP_RIGHT"

	
	-- We may use these variables later to contain the width and height of the screen
	local baseW, baseH
	
	
	-- Declare any OBS objects here and use the releaseAll function to make sure
	-- we don't create any memory leaks
	local transScene, transSceneSource, sourceA, sourceB, groupL, groupR
	local function releaseAll(animating)
		-- If the 'animating' flag is not set then we are finished and we can
		-- reset the busy flag to allow more transitions to be initiated
		if not animating then scrSwitch.busy = false end
		if transScene then obs.obs_scene_release(transScene) end
		if transSceneSource then obs.obs_source_release(transSceneSource) end
		if sourceA then obs.obs_source_release(sourceA) end
		if sourceB then obs.obs_source_release(sourceB) end
		if groupL then obs.obs_source_release(groupL) end
		if groupR then obs.obs_source_release(groupR) end
	end

	
	-- Abort the transition if the current scene or target scene is the special transition scene
	if sceneNameA == transSceneName or sceneNameB == transSceneName then
		obs.script_log(obs.LOG_INFO, format("Aborting - current scene : %s", transSceneName))
		releaseAll()
		return false
	end

	
	-- Get a reference to the transition scene and its source
	transSceneSource = obs.obs_get_source_by_name(transSceneName)
	if not transSceneSource then
		-- Doesn't exist already - create it
		if debug then obs.script_log(obs.LOG_INFO, format("Creating scene : %s", transSceneName)) end
		transScene = obs.obs_scene_create(transSceneName)
		if transScene then transSceneSource = obs.obs_get_source_by_name(transSceneName) end
	else
		-- Exists already
		if debug then obs.script_log(obs.LOG_INFO, format("Found scene : %s", transSceneName)) end
		transScene = obs.obs_scene_from_source(transSceneSource)
		if transScene then obs.obs_scene_addref(transScene) end
	end
	if not transScene or not transSceneSource then
		if debug then obs.script_log(obs.LOG_INFO, format("Error accessing scene : %s", transSceneName)) end
		releaseAll()
		return false
	end

	
	-- Remove any objects in the transition scene
	emptyScene(transScene)

	
	-- We need the target scene (scene B) on the bottom for this effect, so we'll add it first
	sourceB = obs.obs_get_source_by_name(sceneNameB)
	if not sourceB then
		if debug then obs.script_log(obs.LOG_INFO, format("Can't find sceneB : %s", sceneNameB)) end
		releaseAll()
		return false
	end
	-- Add scene B to the transition scene
	if debug then obs.script_log(obs.LOG_INFO, format("Adding sceneB : %s", sceneNameB)) end
	local itemB = obs.obs_scene_add(transScene, sourceB)


	-- Now get a reference to the current scene (scene A) - we'll use it in a moment...
	sourceA = obs.obs_get_source_by_name(sceneNameA)
	if not sourceA then
		if debug then obs.script_log(obs.LOG_INFO, format("Can't find sceneA : %s", sceneNameA)) end
		releaseAll()
		return false
	end


	-- Create the group for the left hand side of the effect
	groupL = obs.obs_get_source_by_name(groupNameL)
	if not groupL then
		if debug then obs.script_log(obs.LOG_INFO, format("Creating group : %s", groupNameL)) end
		groupL = obs.obs_source_create("group", groupNameL, nil, nil)
		if not groupL then
			if debug then obs.script_log(obs.LOG_INFO, format("Can't create group : %s", groupNameL)) end
			releaseAll()
			return false
		end
	end


	-- Add the group to the transition scene
	local groupItemL = obs.obs_scene_add(transScene, groupL)
	if not groupItemL then
		if debug then obs.script_log(obs.LOG_INFO, format("Can't add group to scene : %s", groupNameL)) end
		releaseAll()
		return false
	end


	-- We need the scene reference for the group so we can put things into it
	local groupSceneL = obs.obs_sceneitem_group_get_scene(groupItemL)
	if not groupSceneL then
		if debug then obs.script_log(obs.LOG_INFO, format("Can't access group : %s", groupNameL)) end
		releaseAll()
		return false
	end


	-- Just in case the group already existed - make sure it's empty
	emptyScene(groupSceneL)


	-- Create the group for the right hand side of the effect
	groupR = obs.obs_get_source_by_name(groupNameR)
	if not groupR then
		if debug then obs.script_log(obs.LOG_INFO, format("Creating group : %s", groupNameR)) end
		groupR = obs.obs_source_create("group", groupNameR, nil, nil)
		if not groupR then
			if debug then obs.script_log(obs.LOG_INFO, format("Can't create group : %s", groupNameR)) end
			releaseAll()
			return false
		end
	end


	-- Add the group to the transition scene
	local groupItemR = obs.obs_scene_add(transScene, groupR)
	if not groupItemR then
		if debug then obs.script_log(obs.LOG_INFO, format("Can't add group to scene : %s", groupNameR)) end
		releaseAll()
		return false
	end


	-- We need the scene reference for the group so we can put things into it
	local groupSceneR = obs.obs_sceneitem_group_get_scene(groupItemR)
	if not groupSceneR then
		if debug then obs.script_log(obs.LOG_INFO, format("Can't access group : %s", groupNameR)) end
		releaseAll()
		return false
	end


	-- Just in case the group already existed - make sure it's empty
	emptyScene(groupSceneR)


	-- Add the current scene (scene A) into both groups
	if debug then obs.script_log(obs.LOG_INFO, format("Adding sceneA : %s", sceneNameA)) end
	local itemAL = obs.obs_scene_add(groupSceneL, sourceA)
	local itemAR = obs.obs_scene_add(groupSceneR, sourceA)


	-- Grab the current screen width and height for later use	
	baseW = obs.obs_source_get_width(sourceA)
	baseH = obs.obs_source_get_height(sourceA)
	if debug then obs.script_log(obs.LOG_INFO, format("Canvas : %d x %d", baseW, baseH)) end


	-- Set alignment for each screen half's hinge effect
	obs.obs_sceneitem_set_alignment(groupItemL, obsAlignment.top + obsAlignment.left)
	obs.obs_sceneitem_set_alignment(groupItemR, obsAlignment.top + obsAlignment.right)


	-- crop the left group so that the right half is hidden
	local crp = obs.obs_sceneitem_crop()
	crp.left, crp.top, crp.right, crp.bottom = 0, 0, baseW * 0.5, 0
	obs.obs_sceneitem_set_crop(groupItemL, crp)


	-- crop the right group so that the left half is hidden 
	crp.left, crp.right = baseW * 0.5, 0
	obs.obs_sceneitem_set_crop(groupItemR, crp)

	
	-- switch the normal OBS transition to 'Cut'
	-- (used for switching from current scene to transition scene then to target scene)
	local transitions = obs.obs_frontend_get_transitions()
	local transition = findSource(transitions, "Cut")
	if transition then obs.obs_frontend_set_current_transition(transition) end
	obs.source_list_release(transitions)


	-- record the current scene so that it can be referred to as '_previous'
	prevScene = currentSceneName()


	-- This outer timer block is run in 200 ms - you'll see the number near the bottom, below
	obs.timer_add(


		function()
			-- Cut to the transition scene
			obs.obs_frontend_set_current_scene(transSceneSource)


			obs.script_log(obs.LOG_INFO, "Transition ready to animate")
			

			-- Commands just like those from deck buttons, but with a slightly different format;
			-- no command descriptions and new lines replaced with tabs ( \t )
			-- also a newline character ( \n ) at the end of each command
			cmd = format(
				"animate\ttag=UPDtrans\tscene=%s\titem=%s\tsteps=20\tinterval=10\tr=90\teasing=easein\n"..
				"animate\ttag=UPDtrans\tscene=%s\titem=%s\tsteps=20\tinterval=10\tr=-90\teasing=easein\n",
				transSceneName, groupNameL, transSceneName, groupNameR
			)


			-- Send the commands - just as if they had been received from the mobile app.
			-- This will start the animations
			listen(cmd)


			-- Inner timer : run every 500 ms to check for completion
			obs.timer_add(


				function()


					-- Check if animations are still running
					if (animTag["UPDtrans"] or 0) > 0 then


						-- still busy
						obs.script_log(obs.LOG_INFO, "Transition in progress...")


					else


						-- The final switch to the target scene
						local targetScene = obs.obs_get_source_by_name(sceneNameB)
						if targetScene then
							obs.obs_frontend_set_current_scene(targetScene)
							obs.obs_source_release(targetScene)
						end


						-- Unblock new transitions
						scrSwitch.busy = false


						-- We are done! - remove the timer
						obs.script_log(obs.LOG_INFO, "Transition complete")
						obs.remove_current_callback()


					end
				end,
				500 -- (Inner timer runs every 500 ms)


			)  -- Inner timer


			-- Release all our OBS references, but use the 'animating' flag to let it know we are NOT finished
			releaseAll("animating")


			-- This block only needs to run once to get the animation moving, so remove it
			obs.remove_current_callback()
		end,
		200 -- (Outer timer runs after 200 ms)


	)  -- Outer timer


end